@@ -17,19 +17,15 @@
 #include <malloc.h>
 #include <osi.h>
 #include <rx/rx.h>
-#ifndef DJGPP
 #include <rx/rxkad.h>
-#else
-#include <rx/rxkad.h>
-#endif
-
+#include <afs/unified_afs.h>
 #include "afsd.h"
 
 osi_rwlock_t cm_connLock;
 
 long RDRtimeout = CM_CONN_DEFAULTRDRTIMEOUT;
-long ConnDeadtimeout = CM_CONN_CONNDEADTIME;
-long HardDeadtimeout = CM_CONN_HARDDEADTIME;
+unsigned short ConnDeadtimeout = CM_CONN_CONNDEADTIME;
+unsigned short HardDeadtimeout = CM_CONN_HARDDEADTIME;
 
 #define LANMAN_WKS_PARAM_KEY "SYSTEM\\CurrentControlSet\\Services\\lanmanworkstation\\parameters"
 #define LANMAN_WKS_SESSION_TIMEOUT "SessTimeout"
@@ -90,7 +86,7 @@ void cm_InitReq(cm_req_t *reqp)
 {
 	memset((char *)reqp, 0, sizeof(cm_req_t));
 #ifndef DJGPP
-	reqp->startTime = GetCurrentTime();
+	reqp->startTime = GetTickCount();
 #else
         gettimeofday(&reqp->startTime, NULL);
 #endif
@@ -153,9 +149,10 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     int dead_session;
     long timeUsed, timeLeft;
     long code;
-        
-    osi_Log2(afsd_logp, "cm_Analyze connp 0x%x, code 0x%x",
-             (long) connp, errorCode);
+    char addr[16];
+
+    osi_Log2(afsd_logp, "cm_Analyze connp 0x%p, code 0x%x",
+             connp, errorCode);
 
     /* no locking required, since connp->serverp never changes after
      * creation */
@@ -189,7 +186,7 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     
     /* timeleft - get if from reqp the same way as cmXonnByMServers does */
 #ifndef DJGPP
-    timeUsed = (GetCurrentTime() - reqp->startTime) / 1000;
+    timeUsed = (GetTickCount() - reqp->startTime) / 1000;
 #else
     gettimeofday(&now, NULL);
     timeUsed = sub_time(now, reqp->startTime) / 1000;
@@ -336,31 +333,46 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     else if (errorCode == VNOVOL || errorCode == VMOVED || errorCode == VOFFLINE ||
              errorCode == VSALVAGE || errorCode == VNOSERVICE || errorCode == VIO) 
     {       
-#ifndef DJGPP
-        HANDLE h;
-        char *ptbuf[1];
-        char s[100];
-#endif
         char addr[16];
         char *format;
-
+#ifndef DJGPP
+	DWORD msgID;
+#endif
         switch ( errorCode ) {
         case VNOVOL:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VNOVOL;
+#endif
             format = "Server %s reported volume %d as not attached.";
             break;
         case VMOVED:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VMOVED;
+#endif
             format = "Server %s reported volume %d as moved.";
             break;
         case VOFFLINE:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VOFFLINE;
+#endif
             format = "Server %s reported volume %d as offline.";
             break;
         case VSALVAGE:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VSALVAGE;
+#endif
             format = "Server %s reported volume %d as needs salvage.";
             break;
         case VNOSERVICE:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VNOSERVICE;
+#endif
             format = "Server %s reported volume %d as not in service.";
             break;
         case VIO:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VIO;
+#endif
             format = "Server %s reported volume %d as temporarily unaccessible.";
             break;
         }
@@ -371,16 +383,10 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
                  ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
                  ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
                  ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24)); 
-        osi_Log2(afsd_logp, format, osi_LogSaveString(afsd_logp,addr), fidp->volume);
-        
+
+	osi_Log2(afsd_logp, format, osi_LogSaveString(afsd_logp,addr), fidp->volume);
 #ifndef DJGPP
-        /* Create Event Log message */
-        h = RegisterEventSource(NULL, AFS_DAEMON_EVENT_NAME);
-        sprintf(s, format, addr, fidp->volume);
-        ptbuf[0] = s;
-        ReportEvent(h, EVENTLOG_WARNING_TYPE, 0, 1009, NULL,
-                     1, 0, ptbuf, NULL);
-        DeregisterEventSource(h);
+	LogEvent(EVENTLOG_WARNING_TYPE, msgID, addr, fidp->volume);
 #endif
 
         /* Mark server offline for this volume */
@@ -401,6 +407,35 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
         }
         if ( timeLeft > 2 )
             retry = 1;
+    } else if ( errorCode == VNOVNODE ) {
+	if ( fidp ) {
+	    cm_scache_t * scp;
+	    osi_Log4(afsd_logp, "cm_Analyze passed VNOVNODE cell %u vol %u vn %u uniq %u.",
+		      fidp->cell, fidp->volume, fidp->vnode, fidp->unique);
+
+	    scp = cm_FindSCache(fidp);
+	    if (scp) {
+		cm_scache_t *pscp = NULL;
+
+		if (scp->fileType != CM_SCACHETYPE_DIRECTORY)
+		    pscp = cm_FindSCacheParent(scp);
+
+		lock_ObtainWrite(&cm_scacheLock);
+		cm_RecycleSCache(scp, CM_SCACHE_RECYCLEFLAG_DESTROY_BUFFERS);
+		lock_ReleaseWrite(&cm_scacheLock);
+
+ 		if (pscp) {
+		    if (pscp->cbExpires > 0 && pscp->cbServerp != NULL) {
+ 			lock_ObtainMutex(&pscp->mx);
+ 			cm_DiscardSCache(pscp);
+ 			lock_ReleaseMutex(&pscp->mx);
+ 		    }
+ 		    cm_ReleaseSCache(pscp);
+ 		}
+	    }
+	} else {
+	    osi_Log0(afsd_logp, "cm_Analyze passed VNOVNODE unknown fid.");
+	}
     }
 
     /* RX codes */
@@ -410,26 +445,28 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
          * this is to prevent the SMB session from timing out
          * In addition, we log an event to the event log 
          */
+
+        /* Log server being offline for this volume */
+        sprintf(addr, "%d.%d.%d.%d", 
+                 ((serverp->addr.sin_addr.s_addr & 0xff)),
+                 ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
+                 ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
+                 ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24)); 
+
 #ifndef DJGPP
-        HANDLE h;
-        char *ptbuf[1];
-        char s[100];
-        h = RegisterEventSource(NULL, AFS_DAEMON_EVENT_NAME);
-        sprintf(s, "cm_Analyze: HardDeadTime exceeded.");
-        ptbuf[0] = s;
-        ReportEvent(h, EVENTLOG_WARNING_TYPE, 0, 1009, NULL,
-                     1, 0, ptbuf, NULL);
-        DeregisterEventSource(h);
+	LogEvent(EVENTLOG_WARNING_TYPE, MSG_RX_HARD_DEAD_TIME_EXCEEDED, addr);
 #endif /* !DJGPP */
 	  
         retry = 0;
-        osi_Log0(afsd_logp, "cm_Analyze: hardDeadTime exceeded");
+        osi_Log1(afsd_logp, "cm_Analyze: hardDeadTime exceeded addr[%s]",
+		 osi_LogSaveString(afsd_logp,addr));
     }
     else if (errorCode >= -64 && errorCode < 0) {
         /* mark server as down */
         lock_ObtainMutex(&serverp->mx);
         serverp->flags |= CM_SERVERFLAG_DOWN;
         lock_ReleaseMutex(&serverp->mx);
+	cm_ForceNewConnections(serverp);
         if ( timeLeft > 2 )
             retry = 1;
     }
@@ -480,6 +517,65 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
             case VRESTRICTED       : s = "VRESTRICTED";        break;
             case VRESTARTING       : s = "VRESTARTING";        break;
             case VREADONLY         : s = "VREADONLY";          break;
+            case EAGAIN            : s = "EAGAIN";             break;
+	    case UAEAGAIN          : s = "UAEAGAIN";	       break;
+            case EINVAL            : s = "EINVAL";             break;
+	    case UAEINVAL          : s = "UAEINVAL";	       break;
+            case EACCES            : s = "EACCES";             break;
+	    case UAEACCES 	   : s = "UAECCES";            break;
+	    case ENOENT            : s = "ENOENT"; 	       break;
+	    case UAENOENT          : s = "UAENOENT";           break;
+	    case CM_ERROR_NOSUCHCELL	    : s = "CM_ERROR_NOSUCHCELL";         break; 			
+	    case CM_ERROR_NOSUCHVOLUME	    : s = "CM_ERROR_NOSUCHVOLUME";       break; 			
+	    case CM_ERROR_TIMEDOUT	    : s = "CM_ERROR_TIMEDOUT";           break; 		
+	    case CM_ERROR_RETRY		    : s = "CM_ERROR_RETRY";              break; 
+	    case CM_ERROR_NOACCESS	    : s = "CM_ERROR_NOACCESS";           break; 
+	    case CM_ERROR_NOSUCHFILE	    : s = "CM_ERROR_NOSUCHFILE";         break; 			
+	    case CM_ERROR_STOPNOW	    : s = "CM_ERROR_STOPNOW";            break; 			
+	    case CM_ERROR_TOOBIG	    : s = "CM_ERROR_TOOBIG";             break; 				
+	    case CM_ERROR_INVAL		    : s = "CM_ERROR_INVAL";              break; 				
+	    case CM_ERROR_BADFD		    : s = "CM_ERROR_BADFD";              break; 				
+	    case CM_ERROR_BADFDOP	    : s = "CM_ERROR_BADFDOP";            break; 			
+	    case CM_ERROR_EXISTS	    : s = "CM_ERROR_EXISTS";             break; 				
+	    case CM_ERROR_CROSSDEVLINK	    : s = "CM_ERROR_CROSSDEVLINK";       break; 			
+	    case CM_ERROR_BADOP		    : s = "CM_ERROR_BADOP";              break; 				
+	    case CM_ERROR_BADPASSWORD       : s = "CM_ERROR_BADPASSWORD";        break;         
+	    case CM_ERROR_NOTDIR	    : s = "CM_ERROR_NOTDIR";             break; 				
+	    case CM_ERROR_ISDIR		    : s = "CM_ERROR_ISDIR";              break; 				
+	    case CM_ERROR_READONLY	    : s = "CM_ERROR_READONLY";           break; 			
+	    case CM_ERROR_WOULDBLOCK	    : s = "CM_ERROR_WOULDBLOCK";         break; 			
+	    case CM_ERROR_QUOTA		    : s = "CM_ERROR_QUOTA";              break; 				
+	    case CM_ERROR_SPACE		    : s = "CM_ERROR_SPACE";              break; 				
+	    case CM_ERROR_BADSHARENAME	    : s = "CM_ERROR_BADSHARENAME";       break; 			
+	    case CM_ERROR_BADTID	    : s = "CM_ERROR_BADTID";             break; 				
+	    case CM_ERROR_UNKNOWN	    : s = "CM_ERROR_UNKNOWN";            break; 			
+	    case CM_ERROR_NOMORETOKENS	    : s = "CM_ERROR_NOMORETOKENS";       break; 			
+	    case CM_ERROR_NOTEMPTY	    : s = "CM_ERROR_NOTEMPTY";           break; 			
+	    case CM_ERROR_USESTD	    : s = "CM_ERROR_USESTD";             break; 				
+	    case CM_ERROR_REMOTECONN	    : s = "CM_ERROR_REMOTECONN";         break; 			
+	    case CM_ERROR_ATSYS		    : s = "CM_ERROR_ATSYS";              break; 				
+	    case CM_ERROR_NOSUCHPATH	    : s = "CM_ERROR_NOSUCHPATH";         break; 			
+	    case CM_ERROR_CLOCKSKEW	    : s = "CM_ERROR_CLOCKSKEW";          break; 			
+	    case CM_ERROR_BADSMB	    : s = "CM_ERROR_BADSMB";             break; 				
+	    case CM_ERROR_ALLBUSY	    : s = "CM_ERROR_ALLBUSY";            break; 			
+	    case CM_ERROR_NOFILES	    : s = "CM_ERROR_NOFILES";            break; 			
+	    case CM_ERROR_PARTIALWRITE	    : s = "CM_ERROR_PARTIALWRITE";       break; 			
+	    case CM_ERROR_NOIPC		    : s = "CM_ERROR_NOIPC";              break; 				
+	    case CM_ERROR_BADNTFILENAME	    : s = "CM_ERROR_BADNTFILENAME";      break; 			
+	    case CM_ERROR_BUFFERTOOSMALL    : s = "CM_ERROR_BUFFERTOOSMALL";     break; 			
+	    case CM_ERROR_RENAME_IDENTICAL  : s = "CM_ERROR_RENAME_IDENTICAL";   break; 		
+	    case CM_ERROR_ALLOFFLINE        : s = "CM_ERROR_ALLOFFLINE";         break;          
+	    case CM_ERROR_AMBIGUOUS_FILENAME: s = "CM_ERROR_AMBIGUOUS_FILENAME"; break;  
+	    case CM_ERROR_BADLOGONTYPE	    : s = "CM_ERROR_BADLOGONTYPE";       break; 	    
+	    case CM_ERROR_GSSCONTINUE       : s = "CM_ERROR_GSSCONTINUE";        break;         
+	    case CM_ERROR_TIDIPC            : s = "CM_ERROR_TIDIPC";             break;              
+	    case CM_ERROR_TOO_MANY_SYMLINKS : s = "CM_ERROR_TOO_MANY_SYMLINKS";  break;   
+	    case CM_ERROR_PATH_NOT_COVERED  : s = "CM_ERROR_PATH_NOT_COVERED";   break;    
+	    case CM_ERROR_LOCK_CONFLICT     : s = "CM_ERROR_LOCK_CONFLICT";      break;       
+	    case CM_ERROR_SHARING_VIOLATION : s = "CM_ERROR_SHARING_VIOLATION";  break;   
+	    case CM_ERROR_ALLDOWN           : s = "CM_ERROR_ALLDOWN";            break;             
+	    case CM_ERROR_TOOFEWBUFS	    : s = "CM_ERROR_TOOFEWBUFS";         break; 			
+	    case CM_ERROR_TOOMANYBUFS	    : s = "CM_ERROR_TOOMANYBUFS";        break; 			
             }
             osi_Log2(afsd_logp, "cm_Analyze: ignoring error code 0x%x (%s)", 
                      errorCode, s);
@@ -520,7 +616,7 @@ long cm_ConnByMServers(cm_serverRef_t *serversp, cm_user_t *usersp,
     *connpp = NULL;
 
 #ifndef DJGPP
-    timeUsed = (GetCurrentTime() - reqp->startTime) / 1000;
+    timeUsed = (GetTickCount() - reqp->startTime) / 1000;
 #else
     gettimeofday(&now, NULL);
     timeUsed = sub_time(now, reqp->startTime) / 1000;
@@ -694,7 +790,8 @@ long cm_ConnByServer(cm_server_t *serverp, cm_user_t *userp, cm_conn_t **connpp)
         tcp->refCount = 1;
         lock_ReleaseMutex(&tcp->mx);
     } else {
-        if ((tcp->ucgen < ucellp->gen) ||
+        if ((tcp->flags & CM_CONN_FLAG_FORCE_NEW) ||
+            (tcp->ucgen < ucellp->gen) ||
             (tcp->cryptlevel != (cryptall ? (ucellp->flags & CM_UCELLFLAG_RXKAD ? rxkad_crypt : rxkad_clear) : rxkad_clear)))
         {
             if (tcp->ucgen < ucellp->gen)
@@ -702,6 +799,7 @@ long cm_ConnByServer(cm_server_t *serverp, cm_user_t *userp, cm_conn_t **connpp)
             else
                 osi_Log0(afsd_logp, "cm_ConnByServer replace connection due to crypt change");
             lock_ObtainMutex(&tcp->mx);
+	    tcp->flags &= ~CM_CONN_FLAG_FORCE_NEW;
             rx_DestroyConnection(tcp->callp);
             cm_NewRXConnection(tcp, ucellp, serverp);
             lock_ReleaseMutex(&tcp->mx);
@@ -712,7 +810,7 @@ long cm_ConnByServer(cm_server_t *serverp, cm_user_t *userp, cm_conn_t **connpp)
     lock_ReleaseMutex(&userp->mx);
 
     /* return this pointer to our caller */
-    osi_Log1(afsd_logp, "cm_ConnByServer returning conn 0x%x", (long) tcp);
+    osi_Log1(afsd_logp, "cm_ConnByServer returning conn 0x%p", tcp);
     *connpp = tcp;
 
     return 0;
@@ -747,3 +845,15 @@ cm_GetRxConn(cm_conn_t *connp)
     return rxconn;
 }
 
+void cm_ForceNewConnections(cm_server_t *serverp)
+{
+    cm_conn_t *tcp;
+
+    lock_ObtainWrite(&cm_connLock);
+    for (tcp = serverp->connsp; tcp; tcp=tcp->nextp) {
+	lock_ObtainMutex(&tcp->mx);
+	tcp->flags |= CM_CONN_FLAG_FORCE_NEW;
+	lock_ReleaseMutex(&tcp->mx);
+    }
+    lock_ReleaseWrite(&cm_connLock);
+}
