@@ -47,48 +47,48 @@ extern "C" {
 #define cyMIN_WINDOW  120
 
 rwWindowData awdMain[] = {
-    { IDC_CELL_BORDER, raSizeX             },
-    { IDC_CELL,        raSizeX | raRepaint },
-    { IDC_AFS_ID,      raSizeX | raRepaint },
-    { IDC_SERVERS,     raSizeX | raSizeY,  MAKELONG(cxMIN_SERVER,cyMIN_SERVER) },
-    { IDC_COVERDLG,    raSizeX | raSizeY   },
-    { IDC_ANIMATE,     raMoveX             },
-    { idENDLIST,       0                   }
+    { IDC_CELL_BORDER, raSizeX,			0,	0 },
+    { IDC_CELL,        raSizeX | raRepaint,	0,	0 },
+    { IDC_AFS_ID,      raSizeX | raRepaint,	0,	0 },
+    { IDC_SERVERS,     raSizeX | raSizeY,  	MAKELONG(cxMIN_SERVER,cyMIN_SERVER), 0 },
+    { IDC_COVERDLG,    raSizeX | raSizeY,	0,	0 },
+    { IDC_ANIMATE,     raMoveX,			0,	0 },
+    { idENDLIST,       0,			0,	0 }
  };
 
 rwWindowData awdMainVert[] = {
-    { IDC_CELL_BORDER,     raSizeX                       },
-    { IDC_CELL,            raSizeX | raRepaint           },
-    { IDC_AFS_ID,          raSizeX | raRepaint           },
-    { IDC_SERVERS,         raSizeX, MAKELONG(cxMIN_SERVER,cyMIN_SERVER) },
-    { IDC_SPLITTER_SERVER, raSizeX                       },
-    { IDC_TABS,            raSizeX | raSizeY, MAKELONG(cxMIN_TABS,cyMIN_TABS) },
-    { IDC_COVERDLG,        raSizeX                       },
-    { IDC_ANIMATE,         raMoveX                       },
-    { idENDLIST,           0                             }
+    { IDC_CELL_BORDER,     raSizeX,		0,	0 },
+    { IDC_CELL,            raSizeX | raRepaint,	0,	0 },
+    { IDC_AFS_ID,          raSizeX | raRepaint,	0,	0 },
+    { IDC_SERVERS,         raSizeX, 		MAKELONG(cxMIN_SERVER,cyMIN_SERVER), 0 },
+    { IDC_SPLITTER_SERVER, raSizeX,		0,	0 },
+    { IDC_TABS,            raSizeX | raSizeY, MAKELONG(cxMIN_TABS,cyMIN_TABS), 0 },
+    { IDC_COVERDLG,        raSizeX,             0,	0 },
+    { IDC_ANIMATE,         raMoveX,             0,	0 },
+    { idENDLIST,           0,			0,	0  }
  };
 
 rwWindowData awdMainHorz[] = {
-    { IDC_CELL_BORDER,     raSizeX                       },
-    { IDC_CELL,            raSizeX  | raRepaint          },
-    { IDC_AFS_ID,          raSizeX  | raRepaint          },
-    { IDC_SERVERS,         raSizeY, MAKELONG(cxMIN_SERVER,cyMIN_SERVER) },
-    { IDC_SPLITTER_SERVER, raSizeY                       },
-    { IDC_TABS,            raSizeX | raSizeY, MAKELONG(cxMIN_TABS,cyMIN_TABS) },
-    { IDC_COVERDLG,        raSizeY                       },
-    { IDC_ANIMATE,         raMoveX                       },
-    { idENDLIST,           0                             }
+    { IDC_CELL_BORDER,     raSizeX,			0,	0 },
+    { IDC_CELL,            raSizeX  | raRepaint,	0,	0 },
+    { IDC_AFS_ID,          raSizeX  | raRepaint,	0,	0 },
+    { IDC_SERVERS,         raSizeY, 			MAKELONG(cxMIN_SERVER,cyMIN_SERVER), 0 },
+    { IDC_SPLITTER_SERVER, raSizeY,			0,	0 },
+    { IDC_TABS,            raSizeX | raSizeY, 		MAKELONG(cxMIN_TABS,cyMIN_TABS), 0 },
+    { IDC_COVERDLG,        raSizeY,			0,	0 },
+    { IDC_ANIMATE,         raMoveX,			0,	0 },
+    { idENDLIST,           0,				0,	0 }
  };
 
 rwWindowData awdSplitServer[] = {
-    { IDC_CELL,            raRepaint },
-    { IDC_AFS_ID,          raRepaint },
-    { IDC_SERVERS,         raSizeX | raSizeY,  MAKELONG(cxMIN_SERVER,cyMIN_SERVER) },
-    { IDC_SPLITTER_SERVER, raMoveX | raMoveY },
-    { IDC_TABS,            raMoveX | raMoveY | raSizeXB | raSizeYB, MAKELONG(cxMIN_TABS,cyMIN_TABS) },
-    { IDC_COVERDLG,        raSizeX | raSizeY },
-    { IDC_ANIMATE,         0  },
-    { idENDLIST,           0  }
+    { IDC_CELL,            raRepaint,			0,	0 },
+    { IDC_AFS_ID,          raRepaint,			0,	0 },
+    { IDC_SERVERS,         raSizeX | raSizeY,		MAKELONG(cxMIN_SERVER,cyMIN_SERVER), 0 },
+    { IDC_SPLITTER_SERVER, raMoveX | raMoveY,		0,	0 },
+    { IDC_TABS,            raMoveX | raMoveY | raSizeXB | raSizeYB, MAKELONG(cxMIN_TABS,cyMIN_TABS), 0 },
+    { IDC_COVERDLG,        raSizeX | raSizeY,		0,	0 },
+    { IDC_ANIMATE,         0,				0,	0 },
+    { idENDLIST,           0,				0,	0 }
  };
 
 
@@ -100,9 +100,7 @@ rwWindowData awdSplitServer[] = {
 void Main_OnNotifyFromDispatch (LPNOTIFYSTRUCT lpns);
 
 void Main_OnPreviewPane (BOOL fPreviewNew, BOOL fVertNew, BOOL fStoreView);
-void Main_OnServerView (int lvsNew);
 DWORD WINAPI Main_OnOpenServers_ThreadProc (PVOID lp);
-void Main_OnEndTask_Properties (LPTASKPACKET ptp);
 
 void Main_SubclassServers (HWND hDlg);
 
@@ -160,7 +158,7 @@ BOOL CALLBACK Main_DialogProc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
          // and context-menu requests
          //
          Main_SubclassServers (hDlg);
-         FastList_SetTextCallback (GetDlgItem (hDlg, IDC_SERVERS), GetItemText, (DWORD)((gr.fPreview && !gr.fVert) ? &gr.diHorz.viewSvr : &gr.diVert.viewSvr));
+         FastList_SetTextCallback (GetDlgItem (hDlg, IDC_SERVERS), GetItemText, ((gr.fPreview && !gr.fVert) ? &gr.diHorz.viewSvr : &gr.diVert.viewSvr));
 
          // Create the preview pane and rearrange the children of this dialog.
          // This also fixes the Servers display--was it in Large Icons view?
@@ -256,7 +254,7 @@ BOOL CALLBACK Main_DialogProc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
          break;
 
       case WM_REFRESHED_CREDENTIALS:
-         g.hCreds = (PVOID)lp;
+         g.hCreds = (UINT_PTR)lp;
          UpdateDisplay_Cell(FALSE);
          StartTask (taskREFRESH_CREDS, NULL, g.lpiCell);
          break;
@@ -529,7 +527,7 @@ DWORD WINAPI Main_OnOpenServers_ThreadProc (PVOID lp)
 
 DWORD WINAPI Main_Redraw_ThreadProc (PVOID lp)
 {
-   BOOL fInvalidate = (BOOL)lp;
+   BOOL fInvalidate = (BOOL)(UINT_PTR)lp;
    AfsClass_Enter();
 
    if (g.lpiCell == NULL)
@@ -696,7 +694,7 @@ void Main_OnServerView (int lvsNew)
       FL_RestoreView (hServers, &gr.diVert.viewSvr);
 
    Main_SetServerViewMenus();
-   FastList_SetTextCallback (GetDlgItem (g.hMain, IDC_SERVERS), GetItemText, (DWORD)((gr.fPreview && !gr.fVert) ? &gr.diHorz.viewSvr : &gr.diVert.viewSvr));
+   FastList_SetTextCallback (GetDlgItem (g.hMain, IDC_SERVERS), GetItemText, ((gr.fPreview && !gr.fVert) ? &gr.diHorz.viewSvr : &gr.diVert.viewSvr));
    UpdateDisplay_Servers (FALSE, NULL, 0);
 }
 
@@ -843,7 +841,7 @@ void Main_RearrangeChildren (BOOL fPreviewNew, BOOL fVertNew)
 
    // Now move the children to their new places!
    //
-   size_t nWindows = 1;
+   int nWindows = 1;
    if (GetDlgItem (g.hMain, IDC_COVERDLG))
       ++nWindows;
    if (GetDlgItem (g.hMain, IDC_TABS))
@@ -876,7 +874,7 @@ void Main_RearrangeChildren (BOOL fPreviewNew, BOOL fVertNew)
 
 
 
-static LONG procServers = 0;
+static LONG_PTR procServers = 0;
 
 LRESULT CALLBACK Main_SubclassServersProc (HWND hServers, UINT msg, WPARAM wp, LPARAM lp)
 {
@@ -885,13 +883,13 @@ LRESULT CALLBACK Main_SubclassServersProc (HWND hServers, UINT msg, WPARAM wp, L
    if (procServers == 0)
       rc = DefWindowProc (hServers, msg, wp, lp);
    else
-      rc = CallWindowProc ((WNDPROC)procServers, hServers, msg, wp, lp);
+      rc = (LRESULT) CallWindowProc ((WNDPROC)procServers, hServers, msg, wp, lp);
 
    switch (msg)
       {
       case WM_DESTROY:
          if (procServers != 0)
-            SetWindowLong (hServers, GWL_WNDPROC, procServers);
+            SetWindowLongPtr (hServers, GWLP_WNDPROC, procServers);
          break;
 
       case WM_COMMAND:
@@ -930,8 +928,8 @@ LRESULT CALLBACK Main_SubclassServersProc (HWND hServers, UINT msg, WPARAM wp, L
 void Main_SubclassServers (HWND hDlg)
 {
    HWND hServers = GetDlgItem (hDlg, IDC_SERVERS);
-   procServers = GetWindowLong (hServers, GWL_WNDPROC);
-   SetWindowLong (hServers, GWL_WNDPROC, (LONG)Main_SubclassServersProc);
+   procServers = GetWindowLongPtr (hServers, GWLP_WNDPROC);
+   SetWindowLongPtr (hServers, GWLP_WNDPROC, (LONG_PTR)Main_SubclassServersProc);
 }
 
 
