@@ -73,7 +73,6 @@
 #define SYS_NAME_ID_rs_aix43		 705
 #define SYS_NAME_ID_rs_aix52		 706
 #define SYS_NAME_ID_rs_aix53		 707
-#define SYS_NAME_ID_rs_aix61		 708
 
 #define SYS_NAME_ID_sun3_411		 906
 #define SYS_NAME_ID_sun3x_411		 912
@@ -100,7 +99,6 @@
 #define SYS_NAME_ID_sun4x_58		 939
 #define SYS_NAME_ID_sun4x_59		 940
 #define SYS_NAME_ID_sun4x_510		 941
-#define SYS_NAME_ID_sun4x_511		 942
 
 /* Sigh. If I leave a gap here IBM will do this sequentially. If I don't
    they won't allocate sunx86 IDs at all. So leave a gap and pray. */
@@ -108,7 +106,6 @@
 #define SYS_NAME_ID_sunx86_58		 951
 #define SYS_NAME_ID_sunx86_59		 952
 #define SYS_NAME_ID_sunx86_510		 953
-#define SYS_NAME_ID_sunx86_511		 954
 
 #define SYS_NAME_ID_vax_ul4		1003
 #define SYS_NAME_ID_vax_ul40		1004
@@ -185,9 +182,6 @@
 #define SYS_NAME_ID_i386_fbsd_53        2108
 #define SYS_NAME_ID_i386_fbsd_60        2112
 #define SYS_NAME_ID_i386_fbsd_61        2113
-#define SYS_NAME_ID_i386_fbsd_62        2114
-#define SYS_NAME_ID_i386_fbsd_70        2115
-#define SYS_NAME_ID_i386_fbsd_80        2116
 
 #define SYS_NAME_ID_ia64_linux2		2200
 #define SYS_NAME_ID_ia64_linux22	2201
@@ -219,20 +213,6 @@
 #define SYS_NAME_ID_i386_nbsd21		2525
 #define SYS_NAME_ID_i386_nbsd30		2526
 #define SYS_NAME_ID_amd64_nbsd20	2527
-#define SYS_NAME_ID_i386_nbsd40 2528
-#define SYS_NAME_ID_i386_nbsd50 2529
-#define SYS_NAME_ID_sparc64_nbsd30 2530
-#define SYS_NAME_ID_sparc64_nbsd40 2531
-#define SYS_NAME_ID_sparc64_nbsd50 2532
-#define SYS_NAME_ID_amd64_nbsd30 2533
-#define SYS_NAME_ID_amd64_nbsd40 2534
-#define SYS_NAME_ID_amd64_nbsd50 2535
-#define SYS_NAME_ID_alpha_nbsd30 2536
-#define SYS_NAME_ID_alpha_nbsd40 2537
-#define SYS_NAME_ID_alpha_nbsd50 2538
-#define SYS_NAME_ID_macppc_nbsd30 2539
-#define SYS_NAME_ID_macppc_nbsd40 2540
-#define SYS_NAME_ID_macppc_nbsd50 2541
 
 #define SYS_NAME_ID_i386_obsd31		2600
 #define SYS_NAME_ID_i386_obsd32		2601
@@ -243,8 +223,6 @@
 #define SYS_NAME_ID_i386_obsd37		2606
 #define SYS_NAME_ID_i386_obsd38		2607
 #define SYS_NAME_ID_i386_obsd39		2608
-#define SYS_NAME_ID_i386_obsd40         2609
-#define SYS_NAME_ID_i386_obsd41         2610
 
 #define SYS_NAME_ID_amd64_linux2        2700
 #define SYS_NAME_ID_amd64_linux22       2701
@@ -267,11 +245,6 @@
 
 #define SYS_NAME_ID_i64_w2k             3500
 
-#define SYS_NAME_ID_arm_linux2          3800
-#define SYS_NAME_ID_arm_linux22         3801
-#define SYS_NAME_ID_arm_linux24         3802
-#define SYS_NAME_ID_arm_linux26         3803
-
 /*
  * Placeholder to keep system-wide standard flags since this file is included by all 
  * files (i.e in afs/param.h)
@@ -295,4 +268,6 @@
 #ifdef	AFS_KERBREALM_ENV
 #define	AFS_REALM_SZ		64
 #endif
+/* Specifies the number of equivalent local realm names */
+#define AFS_NUM_LREALMS         4
 #endif /* __AFS_SYSNAMES_INCL_ENV_ */
