@@ -112,7 +112,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ptserver/ptserver.c,v 1.21.2.1 2005/04/15 19:40:43 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ptserver/ptserver.c,v 1.25 2006/02/22 04:07:39 jaltman Exp $");
 
 #include <afs/stds.h>
 #ifdef	AFS_AIX32_ENV
@@ -120,6 +120,8 @@ RCSID
 #endif
 #include <sys/types.h>
 #include <stdio.h>
+#include <fcntl.h>
+#include <sys/stat.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #include <WINNT/afsevent.h>
@@ -164,6 +166,7 @@ int pr_realmNameLen;
 char *pr_realmName;
 
 int restricted = 0;
+int rxMaxMTU = -1;
 
 static struct afsconf_cell info;
 
@@ -241,6 +244,7 @@ main(int argc, char **argv)
     sigaction(SIGABRT, &nsa, NULL);
     sigaction(SIGSEGV, &nsa, NULL);
 #endif
+    osi_audit_init();
     osi_audit(PTS_StartEvent, 0, AUD_END);
 
     /* Initialize dirpaths */
@@ -315,39 +319,88 @@ main(int argc, char **argv)
 	    serverLogSyslogFacility = atoi(arg + 8);
 	}
 #endif
+	else if (strncmp(arg, "-auditlog", alen) == 0) {
+	    int tempfd, flags;
+	    FILE *auditout;
+	    char oldName[MAXPATHLEN];
+	    char *fileName = argv[++a];
+
+#ifndef AFS_NT40_ENV
+	    struct stat statbuf;
+
+	    if ((lstat(fileName, &statbuf) == 0) 
+		&& (S_ISFIFO(statbuf.st_mode))) {
+		flags = O_WRONLY | O_NONBLOCK;
+	    } else 
+#endif
+	    {
+		strcpy(oldName, fileName);
+		strcat(oldName, ".old");
+		renamefile(fileName, oldName);
+		flags = O_WRONLY | O_TRUNC | O_CREAT;
+	    }
+	    tempfd = open(fileName, flags, 0666);
+	    if (tempfd > -1) {
+		auditout = fdopen(tempfd, "a");
+		if (auditout) {
+		    osi_audit_file(auditout);
+		    osi_audit(PTS_StartEvent, 0, AUD_END);
+		} else
+		    printf("Warning: auditlog %s not writable, ignored.\n", fileName);
+	    } else
+		printf("Warning: auditlog %s not writable, ignored.\n", fileName);
+	}
+	else if (!strncmp(arg, "-rxmaxmtu", alen)) {
+	    if ((a + 1) >= argc) {
+		fprintf(stderr, "missing argument for -rxmaxmtu\n");
+		PT_EXIT(1);
+	    }
+	    rxMaxMTU = atoi(argv[++a]);
+	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
+		 (rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
+		printf("rxMaxMTU %d% invalid; must be between %d-%d\n",
+			rxMaxMTU, RX_MIN_PACKET_SIZE,
+			RX_MAX_PACKET_DATA_SIZE);
+		PT_EXIT(1);
+	    }
+	} 
 	else if (*arg == '-') {
 	    /* hack in help flag support */
 
 #if defined(SUPERGROUPS)
 #ifndef AFS_NT40_ENV
 	    printf("Usage: ptserver [-database <db path>] "
+		   "[-auditlog <log path>] "
 		   "[-syslog[=FACILITY]] "
 		   "[-p <number of processes>] [-rebuild] "
 		   "[-groupdepth <depth>] "
-		   "[-restricted] "
+		   "[-restricted] [-rxmaxmtu <bytes>]"
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-default_access default_user_access default_group_access] "
 		   "[-help]\n");
 #else /* AFS_NT40_ENV */
 	    printf("Usage: ptserver [-database <db path>] "
+		   "[-auditlog <log path>] "
 		   "[-p <number of processes>] [-rebuild] "
 		   "[-default_access default_user_access default_group_access] "
-		   "[-restricted] "
+		   "[-restricted] [-rxmaxmtu <bytes>]"
 		   "[-groupdepth <depth>] " "[-help]\n");
 #endif
 #else
 #ifndef AFS_NT40_ENV
 	    printf("Usage: ptserver [-database <db path>] "
+		   "[-auditlog <log path>] "
 		   "[-syslog[=FACILITY]] "
 		   "[-p <number of processes>] [-rebuild] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-default_access default_user_access default_group_access] "
-		   "[-restricted] "
+		   "[-restricted] [-rxmaxmtu <bytes>]"
 		   "[-help]\n");
 #else /* AFS_NT40_ENV */
 	    printf("Usage: ptserver [-database <db path>] "
+		   "[-auditlog <log path>] "
 		   "[-default_access default_user_access default_group_access] "
-		   "[-restricted] "
+		   "[-restricted] [-rxmaxmtu <bytes>]"
 		   "[-p <number of processes>] [-rebuild] " "[-help]\n");
 #endif
 #endif
@@ -461,6 +514,10 @@ main(int argc, char **argv)
     /* Disable jumbograms */
     rx_SetNoJumbo();
 
+    if (rxMaxMTU != -1) {
+	rx_SetMaxMTU(rxMaxMTU);
+    }
+
     tservice =
 	rx_NewService(0, PRSRV, "Protection Server", sc, 3,
 		      PR_ExecuteRequest);
